import re
from argparse import ArgumentParser
from difflib import SequenceMatcher
#
def read_nmls( fin, tolower ):
  #
  # function to read namelist files and construct lists of named namelists
  # and their constituent variables (the latter being a list of lists)
  # If tolower is True then all names are converted to lowercase.
  # NEMO conventions are assumed here. In particular:
  # 1. namelists are started with &name_of_namelist appearing as the first non-whitespace
  #    on a line (and not followed by anything other than an optional comment)
  # 2. Only one variable is set on each line
  # 3. namelists are closed with / appearing as the first non-whitespace on a line
  # Compliance with these conventions is not checked.
  #
  try:
   f = open(fin,'r')
  except:
   print('Unable to open ',fin)
   exit()
  nmls   = []
  nvars  = []
  linnos = []
  inside = 0
  blank = 0
  stripa=re.compile('\s*&')       # match whitespace up to leading ampersand
  stripc=re.compile('\s*!.*$')    # match whitespace up to leading comment marker
  stripe=re.compile('\s*=.*$')    # match = (with preceding whitespace) and anything following
  lc=0
  for line in f:
    lc = lc + 1
    nm = re.match(r'^\S*&',line)
    if nm != None:                # Found a new namelist; extract and store name
      #print( stripa.sub('',stripc.sub('',line) ) )
      inside = 1
      if tolower: line = line.lower()
      nmls.append(stripa.sub('',stripc.sub('',line.strip()) ))
      newl = []                   # new temporary list
      lins = []                   # new temporary list
    if nm == None and inside == 1 and re.match(r'^\s*!', line) == None :
      # still inside namelist and not a comment
      if re.match(r'^\s*$', line) != None :
        # ignore blank lines
        blank = blank + 1
      elif re.match(r'^\s*/', line) == None :
        # It is not the closing slash, therefore extract and store variable name
        if tolower: line = line.lower()
        #print( stripe.sub('',stripc.sub('',line) ) )
        newl.append(stripe.sub('',stripc.sub('',line.strip()) ))
        lins.append(lc)
      else :
        # Reached the end of this particular namelist. Append temporary list to list of lists
        inside = 0
        nvars.append(newl)
        linnos.append(lins)
  f.close()
  return nmls, nvars, linnos

def main():
    parser = ArgumentParser(description=
    """
    Check for consistency between cfg namelists and ref versions
     e.g. python nemo_nml_check.py --cfg <cfg namelist> --ref  <ref namelist> --match <closeness factor>
    """)
    parser.add_argument('--cfg', '-c',dest='cfgfile',help='configuration namelist', nargs= '*',default='namelist_cfg')
    parser.add_argument('--ref', '-r',dest='reffile',help='reference namelist', nargs= '*',default='namelist_ref')
    parser.add_argument('--match', '-m',dest='mfactor',help='closeness factor (0-1)', nargs= '*',default='0.75')
    args = parser.parse_args()
    #
    mfact = float(''.join(args.mfactor))
    nmls, nvars, lnos    = read_nmls(''.join(args.reffile), True )    # Construct lists from the   reference   namelist
    cnmls, cnvars, clnos = read_nmls(''.join(args.cfgfile), False)    # Construct lists from the configuration namelist
    #
    # for each non-empty, named namelist in the configuration namelist file: find the equivalent namelist in the
    # reference set and confirm that a match for each variable exists.
    # Checks are performed on strings converted to lowercase. This was done on read for the reference set but
    # the configuration set has been keep in original form so that reports match the file contents.
    #
    nomatch = 0
    for k, nl in enumerate(cnmls):
      #print(k, nl)
      if len(cnvars[k]) > 0:
        for rk, rnl in enumerate(nmls):
          if rnl == nl.lower() :
           for nv, v in enumerate(cnvars[k]):
             #print(k,v,rnl)
             if v.lower() not in nvars[rk] :
               if nomatch == 0 :
                 print('No match in matching blocks for:                           |  Possible close match:')
                 nomatch = 1
               print( '{:<22} in {:<12} (at line no. {:4d})'.format(v,nl,clnos[k][nv]), end='  |  ')
               rapprox=0.0
               for nk2, nv2 in enumerate(nvars[rk]):
                 r=SequenceMatcher(None, v, nv2).ratio()
                 if r >= rapprox:
                   (rapprox, vapprox) = (r, nv2)
               if rapprox > mfact : 
                 print( vapprox )
               else :
                 print(" ")
    
#

if __name__ == '__main__':
    main()
